;=========================================================================
; DIMCL.lsp    Dimension Centerline   (Command function for AutoCAD) 
; (c) Copyright 2001 TimeSavers for CAD
; from: www.timesaversforcad.com
;-------------------------------------------------------------------------
; Description:
;
; Changes linetype of an Associative Dimension extension line.
; Function actually modifies length of extension line<s> to
; shortest length, draws LINE in place of previous extension
; line<s> with specified linetype.
;
; Command options:
;  "L" to change linetype - sets global DCL_T
;  "S" to change ltscale - sets global DCLT_S
;============================================================================
(defun c:dimcl (/ *dc:err* c_e o_m g ed c1 p0 p10 p13 p14 a# arad p1 p2 ss ssl
                  fp1 fp2 dst ldst lyr flyr dxo dxe tdd l_t l1 l2)

  ;-ERROR HANDLING
  (defun *dc:err* (m)
    (or (member m '("Function cancelled" "quit / exit abort" "console break"))
        (prompt (strcat "\n< " m " >\n"))
    )
    (setvar "osmode" o_m)(setvar "cmdecho" c_e)
    (setq *error* *e*)(princ)
  )

  ;-GET VALUES
  (setq *e* (cond (*e*)(*error*))
        *error* *dc:err*
        c_e (getvar "cmdecho")
        o_m (getvar "osmode")
  )
  (setvar "cmdecho" 0)
  (or dcl_t (setq dcl_t "center2"))
  (or (tblsearch "LTYPE" dcl_t)
      (command "linetype" "l" dcl_t "acad.lin" "")
      (tblsearch "LTYPE" dcl_t)
      (prompt (strcat "\nDefault linetype " (strcase dcl_t) " not valid."))
      (setq dcl_t nil)
  )
  (or (= (type dclt_s) 'REAL)(setq dclt_s 1.0000))

  ;-MAIN FUNCTION
  (while (and dcl_t dclt_s
              (not (command "undo" "e" "undo" "g"))
              (setvar "osmode" 512)
              (setq ldst 0 ss nil g T)
              (while (and g
                          (or (initget "Ltype ltScale  ")
                              (setq c1 (getpoint (strcat "\nLtype: " (strcase dcl_t) " / ltScale: " (rtos dclt_s 2) " /<Pick Extension Line>: ")))
                          )
                     )
                (cond ((= c1 "Ltype")
                        (while (and (/= "" (setq l_t (getstring (strcat "\nLinetype <" dcl_t ">: "))))
                                    (not (or (tblsearch "LTYPE" l_t)
                           	     (command "linetype" "L" l_t "acad.lin" "")
                                             (tblsearch "LTYPE" l_t)
                                         )
                                    )
                               )
                        )
                        (or (= l_t "")(setq dcl_t l_t))
                      )
                      ((= c1 "ltScale")
                        (if (setq lt_s (getreal (strcat "\nLinetype Scale <" (rtos dclt_s 2) ">: ")))
                          (setq dclt_s lt_s)
                        )
                      )
                      ((setq ss (ssget "C" c1 c1 '((0 . "DIMENSION"))))(setq g nil))
                      ((setq ssl (ssget "C" c1 c1 '((0 . "LINE"))))
                        (setq ssl (ssname ssl 0) ed (entget ssl) g nil)
                      )
                      ((princ "0 found."))
                )
                (or ss ssl)
              )
         )
    (setq g 0)
    (setvar "osmode" 0)
    (while (and ss (< g (sslength ss)))
      (setq ed (entget (ssname ss g))
            p10 (cdr (assoc 10 ed))
            p13 (cdr (assoc 13 ed))
            p14 (cdr (assoc 14 ed))
            arad (cdr (assoc 50 ed))
            lyr (cdr (assoc 8 ed))
            p0 (cond ((= arad 0)(cons (car p13)(cdr p10)))
                     ((= arad >90)(cons (car p10)(cdr p13)))
                     ((inters p10 (polar p10 arad 1.0) p13 (polar p13 (+ arad (* 0.5 pi)) 1.0) nil))
               )
            tdd (tblsearch "dimstyle" (cdr (assoc 3 ed)))
            dxo (* (cdr (assoc 42 tdd))(getvar "dimscale"))
            dxe (* (cdr (assoc 44 tdd))(getvar "dimscale"))
            l1 (+ (distance c1 p10)(distance c1 p14))
            l2 (+ (distance c1 p0)(distance c1 p13))
      )
      (if (cond ((equal l1 (distance p10 p14) 0.001)(setq p1 p10 p2 p14 a# 14))
                ((equal l2 (distance p0 p13) 0.001)(setq p1 p0 p2 p13 a# 13))
          )
        (progn
          (entmod (list (assoc -1 ed)(cons a# (if (> dxo dxe)(polar p1 (angle p1 p2)(- dxo dxe)) p1))))
          (if (> (setq dst (distance p1 p2)) ldst)
            (setq ldst dst fp1 p1 fp2 p2 flyr lyr)
          )
        )
        (prompt "Must be an Extension line.")
      )
      (setq g (1+ g))
    )
    (if ssl
      (progn
        (setq fp1 (cdr (assoc 10 ed))
              fp2 (cdr (assoc 11 ed))
              flyr (cdr (assoc 8 ed))
        )
        (entdel ssl)(setq ssl nil)
      )
    )
    (if fp1  ;found at least 1 assoc. dimension extension line or single line
      (entmake (list '(0 . "LINE")
                     (cons 8 flyr)
                     (cons 6 dcl_t)
                     (cons 48 dclt_s)
                     (cons 10 (if ss (polar fp1 (angle fp2 fp1) dxe) fp1))
                     (cons 11 (if ss (polar fp2 (angle fp2 fp1) dxo) fp2))
               )
      )
    )
  )

  ;-RETURN VARIABLES
  (setvar "cmdecho" c_e)(setvar "osmode" o_m)
  (setq *error* *e*)
  (princ)
)
;-END FILE
